/* Set the backlight level on a toshiba TECRA S1.
   Copyright ( C ) 2005 Thorsten Zachmann <t.zachmann@zagge.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or ( at your option ) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <sys/io.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#define BUFFER_SIZE        0x20

#define SET_LCD_BRIGHTNESS 0xa2
#define GET_LCD_BRIGHTNESS 0xa3


void smi_command( int function,
                  int inputbuffer[BUFFER_SIZE],
                  int inputsize,
                  int outputbuffer[BUFFER_SIZE] )
{
    int counter = 0;
    int a = 0;
    int input;
    int x;
    int y;
    int smi;

    if ( iopl(3) != 0 )
    {
        printf("iopl(3) failed. Please run as root.\n");
        exit(1);
    }

    do
    {
      a = counter;
      a += 0x60;

      outb( a, 0x72 );
      if ( counter < inputsize )
      {
          input = inputbuffer[counter];
      }
      else
      {
          input = 0;
      }
      outb( input, 0x73 );
      counter++;
    } while ( counter < BUFFER_SIZE );

    outl( 0x8000f840 , 0xcf8 );
    x = inb( 0xcfc );
    x = x & 0xff80;
    x += 0x2c;

    y = inl(x);

    outl( 0, x );

    smi = function;
    smi = smi << 8;
    smi |= 0xe4;
    outw( smi, 0xb2 );

    outl( y, x );

    counter = 0;
    do
    {
      a = counter;
      a += 0x60;
      outb( a, 0x72 );
      input = inb( 0x73 );
      outputbuffer[counter] = input;
      counter++;
    } while ( counter < BUFFER_SIZE );

    iopl(0);
}

void set_brightness( int brightness )
{
    int inputbuffer[BUFFER_SIZE];
    int outputbuffer[BUFFER_SIZE];

    if ( brightness < 0 || brightness > 7 )
        return;

    inputbuffer[0] = brightness;
    smi_command( SET_LCD_BRIGHTNESS, inputbuffer, 1, outputbuffer );
}

int get_brightness()
{
    int inputbuffer[BUFFER_SIZE];
    int outputbuffer[BUFFER_SIZE];

    smi_command( GET_LCD_BRIGHTNESS, inputbuffer, 0, outputbuffer );
    return outputbuffer[0];
}


void usage( char * programname )
{
    printf( "usage: %s [-h] [-s level] [-g]\n", programname );
    printf( "-g        get the LCD brightness level\n" );
    printf( "-h        print this message\n" );
    printf( "-s level  set the LCD brightness to level (level 0-7)\n" );
}

int main( int argc, char *argv[] )
{
    int op;
    int s_brightness = -1;
    int g_brightness = 0;

    while (( op = getopt( argc, argv, "hgs:") ) != EOF )
    {
        switch ( op )
        {
            case 'g':
                g_brightness = 1;
                break;
            case 'h':
                usage( argv[0] );
                exit( 0 );
                break;
            case 's':
                s_brightness = atoi( optarg );
                if ( s_brightness > 7 || s_brightness < 0 )
                {
                    s_brightness = 4;
                }
                break;
            default:
                usage( argv[0] );
                exit( 1 );
                break;
        }
    }

    if ( g_brightness == 1 )
    {
        printf( "brightness level: %d\n", get_brightness() );

    }

    if ( s_brightness != -1 )
    {
        set_brightness( s_brightness );
        printf( "brightness level: %d\n", get_brightness() );
    }
}
